from __future__ import annotations

from gopay.api import GoPay
from gopay.models import GopayConfig
from gopay.payments import Payments


def payments(config: dict, services: dict | None = None) -> Payments:
    """
    Recommended way of initating the GoPay SDK. This methods handles configuration
    validation and if needed, conversion from camelCase to snake_case
    """
    # If any of the config keys are camelCase, convert them to snake_case
    for key in tuple(config.keys()):
        if key == "clientId":
            config["client_id"] = config[key]
            del config[key]
        elif key == "clientSecret":
            config["client_secret"] = config[key]
            del config[key]
        elif key == "gatewayUrl":
            config["gateway_url"] = config[key]
            del config[key]

    # Use Pydantic to validate the config object
    config_model = GopayConfig.parse_obj(config)
    config = config_model.dict()

    # Create and return the Payments and GoPay objects
    gopay = GoPay(config, services or {})
    return Payments(gopay)



from __future__ import annotations

from dataclasses import dataclass, field
from urllib.parse import urlsplit, urlunsplit

from gopay.enums import ContentType, Language
from gopay.http import ApiClient, Request, Response


@dataclass
class GoPay:
    """
    This class saves the basic configuration, sets the base_url and passes requests
    to the API Client
    """

    config: dict
    services: dict = field(default_factory=dict)
    base_url: str = field(default="", init=False)
    api_client: ApiClient = field(init=False)

    def __post_init__(self):
        # Make sure URL will be in the form of example.com/api
        urlparts = urlsplit(self.config["gateway_url"])
        self.base_url = urlunsplit((urlparts.scheme, urlparts.netloc, "/api", "", ""))
        
        # Prepare
        api_client_config = {
            "client_id": self.config["client_id"],
            "client_secret": self.config["client_secret"],
            "gateway_url": self.base_url,
            "scope": self.config["scope"],
        }

        # Add optional parameters if found
        if (timeout := self.config.get("timeout")) is not None:
            api_client_config.update({"timeout": timeout})

        if (logger := self.services.get("logger")) is not None:
            api_client_config.update({"logger": logger})

        if (cache := self.services.get("cache")) is not None:
            api_client_config.update({"cache": cache})


        # Create the API client
        self.api_client = ApiClient(
            **api_client_config
        )

    def call(
        self,
        method: str,
        path: str,
        content_type: ContentType | None = None,
        body: dict | None = None,
    ) -> Response:
        """
        Sets some default headers and passes requests to the API Client
        """
        # Build the request
        request = Request(
            method=method, path=path, content_type=content_type, body=body
        )

        # Add some default headers
        request.headers = {
            "Accept": "application/json",
            "User-Agent": "GoPay Python SDK",
            "Accept-Language": "cs-CZ"
            if self.config["language"] in [Language.CZECH, Language.SLOVAK]
            else "en-US",
        }
        if content_type is not None:
            request.headers["Content-Type"] = content_type

        return self.api_client.send_request(request)



from __future__ import annotations

import logging
from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Any, Callable

if TYPE_CHECKING:
    from gopay.http import AccessToken, Request, Response

LoggerType = Callable[["Request", "Response"], Any]


class AbstractCache(ABC):
    """
    Abstract class for implementing custom caches used to cache the token
    """

    @abstractmethod
    def get_token(self, key: str) -> AccessToken:
        """
        Fetch a token with the specified key from the cache
        """
        ...

    @abstractmethod
    def set_token(self, key: str, token: AccessToken) -> None:
        """
        Save the token to the cache under the specified key
        """
        ...


def default_logger(request: Request, response: Response):
    """
    Logs the HTTP Request and Response when communicating with GoPay using logging.debug
    """
    logging.debug(f"GoPay HTTP Request: {request}")
    logging.debug(f"GoPay HTTP Response: {response}")


@dataclass
class DefaultCache(AbstractCache):
    """
    Default cache implementation just keeps the tokens in memory, it's only uselful
    as long as the object instance is alive
    """

    tokens: dict[str, AccessToken] = field(default_factory=dict, init=False)

    def get_token(self, key: str) -> AccessToken | None:
        """
        Fetch token from memory
        """
        return self.tokens.get(key)

    def set_token(self, key: str, token: AccessToken) -> None:
        """
        Save token to memory
        """
        self.tokens[key] = token



from __future__ import annotations

from dataclasses import dataclass

from gopay.api import GoPay, Response
from gopay.enums import ContentType, Currency


@dataclass
class Payments:
    """
    Handles the API calls. Maps inidividual API Calls to Python methods and adds some
    defaults
    """

    gopay: GoPay

    def create_payment(self, payment: dict) -> Response:
        """
        https://doc.gopay.com#payment-creation
        """

        # Add defaults for goid and lang if not present
        if "target" not in payment:
            payment.update(
                {"target": {"type": "ACCOUNT", "goid": self.gopay.config["goid"]}}
            )
        if "lang" not in payment:
            payment.update({"lang": self.gopay.config["language"]})
        return self.gopay.call("POST", "/payments/payment", ContentType.JSON, payment)

    def get_status(self, payment_id: str | int) -> Response:
        """
        https://doc.gopay.com#payment-inquiry
        """
        return self.gopay.call("GET", f"/payments/payment/{payment_id}")

    def refund_payment(self, payment_id: int | str, amount: int) -> Response:
        """
        https://doc.gopay.com#payment-refund
        """
        return self.gopay.call(
            "POST",
            f"/payments/payment/{payment_id}/refund",
            ContentType.FORM,
            {"amount": amount},
        )

    def create_recurrence(self, payment_id: int | str, payment: dict) -> Response:
        """
        https://doc.gopay.com#creating-a-recurrence
        """
        return self.gopay.call(
            "POST",
            f"/payments/payment/{payment_id}/create-recurrence",
            ContentType.JSON,
            payment,
        )

    def void_recurrence(self, payment_id: int | str) -> Response:
        """
        https://doc.gopay.com#void-a-recurring-payment
        """
        return self.gopay.call(
            "POST", f"/payments/payment/{payment_id}/void-recurrence"
        )

    def capture_authorization(self, payment_id: int | str) -> Response:
        """
        https://doc.gopay.com#capturing-a-preauthorized-payment
        """
        return self.gopay.call("post", f"/payments/payment/{payment_id}/capture")

    def capture_authorization_partial(
        self, payment_id: int | str, payment: dict
    ) -> Response:
        """
        https://doc.gopay.com#partially-capturing-a-preauthorized-payment
        """
        return self.gopay.call(
            "POST",
            f"/payments/payment/{payment_id}/capture",
            ContentType.JSON,
            payment,
        )

    def void_authorization(self, payment_id: int | str) -> Response:
        """
        https://doc.gopay.com#voiding-a-preauthorized-payment
        """
        return self.gopay.call(
            "POST", f"/payments/payment/{payment_id}/void-authorization"
        )

    def get_card_details(self, card_id: int | str) -> Response:
        """
        https://doc.gopay.com#payment-card-inquiry
        """
        return self.gopay.call("GET", f"/payments/cards/{card_id}")

    def delete_card(self, card_id: int | str) -> Response:
        """
        https://doc.gopay.com#payment-card-deletion
        """
        return self.gopay.call("DELETE", f"/payments/cards/{card_id}")

    def get_payment_instruments(self, goid: int | str, currency: Currency) -> Response:
        """
        https://doc.gopay.com#available-payment-methods-for-a-currency
        """
        return self.gopay.call(
            "GET", f"/eshops/eshop/{goid}/payment-instruments/{currency}"
        )

    def get_payment_instruments_all(self, goid: int | str) -> Response:
        """
        https://doc.gopay.com#all-available-payment-methods
        """
        return self.gopay.call("GET", f"/eshops/eshop/{goid}/payment-instruments")

    def get_account_statement(self, statement_request: dict) -> Response:
        """
        https://doc.gopay.com#account-statement
        """
        return self.gopay.call(
            "POST", "/accounts/account-statement", ContentType.JSON, statement_request
        )

    def refund_payment_eet(self, payment_id: int | str, payment_data: dict) -> Response:
        return self.gopay.call(
            "POST",
            f"/payments/payment/{payment_id}/refund",
            ContentType.JSON,
            payment_data,
        )

    def get_history_of_refunds(self, payment_id: int | str) -> Response:
        """
        https://doc.gopay.com/#history-of-refunds
        """
        return self.gopay.call("GET", f"/payments/payment/{payment_id}/refunds")

    def get_eet_receipt_by_payment_id(self, payment_id: int | str) -> Response:
        return self.gopay.call("GET", f"/payments/payment/{payment_id}/eet-receipts")

    def find_eet_receipts_by_filter(self, filter: dict) -> Response:
        return self.gopay.call("POST", "/eet-receipts", ContentType.JSON, filter)

    @property
    def get_embedjs_url(self) -> str:
        return self.gopay.base_url[:-4] + "/gp-gw/js/embed.js"
